import javax.media.*;
import javax.media.format.*;
import com.sun.image.codec.jpeg.*;
import java.awt.Dimension;
import java.io.FileInputStream;
import java.awt.image.BufferedImage;

/** @author Horst Eidenberger (hme@ims.tuwien.ac.at), adaptation: Heinrich Hussmann
 *  @version 1
 **/

 
public class logoCodec implements Effect {

    private Format inputFormat;
    private Format outputFormat;
    private Format[] inputFormats;
    private Format[] outputFormats;

    private logoCodecControl control = new logoCodecControl();
    private int [] logoArray    = null;

    private String logoFile     = "logo.jpg";
    private int logoWidth       = 60;
    private int logoHeight      = 80;
    private int offsetX         = 10;
    private int offsetY         = 10;
    private double alphaChannel = 1.0;


    public void open() { 
    
       try {
          JPEGImageDecoder decoder=JPEGCodec.createJPEGDecoder(new FileInputStream(logoFile));
          BufferedImage logoImg=decoder.decodeAsBufferedImage();
          logoArray = logoImg.getRGB(0, 0, logoWidth, logoHeight, null, 0, logoWidth);
       } catch (Exception e) {
          System.out.println("Exception: " + e);
          System.exit(-1);
       }
    }

    public void close() {}

    public void reset() {
       close();
       open();
    }

    public synchronized int process(Buffer in, Buffer out) {

      out.copy(in);
      int [] data = (int[]) out.getData();

      RGBFormat inFormat = (RGBFormat)in.getFormat();
      int redMask = inFormat.getRedMask();
      int greenMask = inFormat.getGreenMask();
      int blueMask = inFormat.getBlueMask();
      
      Dimension inSize = inFormat.getSize();
      int w = (int)inSize.getWidth();
      int h = (int)inSize.getHeight();

      int x,y, offBuffer,offLogo, pixelData;
      
      for(y=0; y<logoHeight; y++) {
         for(x=0; x<logoWidth; x++) {
            offBuffer = (y+offsetY)*w + offsetX + x;
            offLogo = (y * logoWidth + x);
            
            pixelData =  ((int)((data[offBuffer] & redMask)   * (1-alphaChannel) + 
                         (logoArray[offLogo] & 0x000000ff)    * (alphaChannel))) & redMask;
            pixelData += ((int)((data[offBuffer] & greenMask) * (1-alphaChannel) + 
                         (logoArray[offLogo] & 0x0000ff00)    * (alphaChannel))) & greenMask;
            pixelData += ((int)((data[offBuffer] & blueMask)  * (1-alphaChannel) + 
                         (logoArray[offLogo] & 0x00ff0000)    * (alphaChannel))) & blueMask;

            data[offBuffer] = pixelData;                                    
         }
      }

      return BUFFER_PROCESSED_OK;
    }

    
    public synchronized Object getControl(String controlType) {
      return(control);
    }

    public synchronized Object[] getControls() {
      Object[] controls = { control };
      return(controls);
    }

    private class logoCodecControl implements logoCodecControlIF {

       public java.awt.Component getControlComponent() {
          return(null);
       }

       public String getLogoFileName() {
          return(logoFile);
       }
       public void setLogoFileName(String fileName) {
          logoFile=fileName;
          reset();
       }
       
       public int getLogoWidth() {
          return(logoWidth);
       }
       public void setLogoWidth(int w) {
          logoWidth=w;
          reset();
       }
       
       public int getLogoHeight() {
          return(logoHeight);
       }
       public void setLogoHeight(int h) {
          logoHeight=h;
          reset();
       }
       
       public int getXOffset() {
          return(offsetX);
       }
       public void setXOffset(int ox) {
          offsetX=ox;
          reset();
       }

       public int getYOffset() {
          return(offsetY);
       }
       public void setYOffset(int oy) {
          offsetY=oy;
          reset();
       }

       public double getAlphaChannel() {
          return(alphaChannel);
       }
       public void setAlphaChannel(double ac) {
          alphaChannel=ac;
          reset();
       }
    }
    

    public logoCodec() {
        inputFormats = new Format[] {
            new RGBFormat(null,Format.NOT_SPECIFIED,Format.intArray,Format.NOT_SPECIFIED,32,
                          Format.NOT_SPECIFIED,Format.NOT_SPECIFIED,Format.NOT_SPECIFIED)
        };

        outputFormats = new Format[] {
            new RGBFormat(null,Format.NOT_SPECIFIED,Format.intArray,Format.NOT_SPECIFIED,32,
                          Format.NOT_SPECIFIED,Format.NOT_SPECIFIED,Format.NOT_SPECIFIED)
        };
    }

    public Format[] getSupportedInputFormats() {
    	return inputFormats;
    }

    public Format [] getSupportedOutputFormats(Format input) {
        if (input==null) return(outputFormats);
        if (input instanceof RGBFormat)
           return(new Format[] {   
              input,
              outputFormats[0]
           });
        else return(outputFormats);
    }

    public Format setInputFormat(Format input) {
        if (input instanceof RGBFormat) inputFormat = (RGBFormat)input;
        else inputFormat = (RGBFormat)inputFormats[0];
        return(inputFormat);
    }

    public Format setOutputFormat(Format output) {
        if (output instanceof RGBFormat) outputFormat = (RGBFormat)output;
        else outputFormat = (RGBFormat)outputFormats[0];
        return(outputFormat);
    }

    public String getName() {
        return "logo blending codec";
    }
}
